public inbox for gcc-patches@gcc.gnu.org
 help / color / mirror / Atom feed
* [PATCH] Fix function numbering in vmsdbgout.c
@ 2010-06-26 21:41 Douglas B Rupp
  2010-06-28 17:30 ` Richard Henderson
  0 siblings, 1 reply; 6+ messages in thread
From: Douglas B Rupp @ 2010-06-26 21:41 UTC (permalink / raw)
  To: Jim Wilson, Richard Henderson; +Cc: gcc-patches

[-- Attachment #1: Type: text/plain, Size: 83 bytes --]

This patch fixes a bug in VMS Debug format with function numbering.

OK to commit?

[-- Attachment #2: 10.3debugalpha.dif.txt --]
[-- Type: text/plain, Size: 5145 bytes --]

2010-06-26  Douglas B Rupp  <rupp@gnat.com>

	* vmsdbgout.c (func_table): Replace with func{nam,num}_tables.
	(funcnam_table): New static table.
	(funcnum_table): New static table.
	(write_rtnbeg): Write value saved in funcnum_table.
	(write_rtnend): Write value saved in funcnum_table.
	(vmsdbgout_begin_function): Save current function info in
	func{nam,num}_tables. Resize as necessary.
	(vmsdbgout_init): Initialize func{nam,num}_tables. Minor reformatting.

--- gcc/vmsdbgout.c	2010-06-16 12:40:46.000000000 -0700
+++ gcc/vmsdbgout.c	2010-06-25 23:25:23.000000000 -0700
@@ -106,26 +106,12 @@ static unsigned int file_info_table_in_u
    table.  */
 #define FILE_TABLE_INCREMENT 64
 
-/* A structure to hold basic information for the VMS end
-   routine.  */
-
-typedef struct vms_func_struct
-{
-  const char *vms_func_name;
-  unsigned funcdef_number;
-}
-vms_func_node;
-
-typedef struct vms_func_struct *vms_func_ref;
-
+static char **funcnam_table;
+static unsigned int *funcnum_table;
 static unsigned int func_table_allocated;
 static unsigned int func_table_in_use;
 #define FUNC_TABLE_INCREMENT 256
 
-/* A pointer to the base of a table that contains frame description
-   information for each routine.  */
-static vms_func_ref func_table;
-
 /* Local pointer to the name of the main input file.  Initialized in
    vmsdbgout_init.  */
 static const char *primary_filename;
@@ -805,9 +665,8 @@ write_rtnbeg (int rtnnum, int dosizeonly
   char label[MAX_ARTIFICIAL_LABEL_BYTES];
   DST_ROUTINE_BEGIN rtnbeg;
   DST_PROLOG prolog;
-  vms_func_ref fde = &func_table[rtnnum];
 
-  rtnname = fde->vms_func_name;
+  rtnname = funcnam_table [rtnnum];
   rtnnamelen = strlen (rtnname);
   rtnentryname = concat (rtnname, "..en", NULL);
 
@@ -878,7 +737,8 @@ write_rtnbeg (int rtnnum, int dosizeonly
       totsize += write_debug_header (&prolog.dst_a_prolog_header, "prolog",
 				     dosizeonly);
 
-      ASM_GENERATE_INTERNAL_LABEL (label, FUNC_PROLOG_LABEL, fde->funcdef_number);
+      ASM_GENERATE_INTERNAL_LABEL (label, FUNC_PROLOG_LABEL,
+				   funcnum_table [rtnnum]);
       totsize += write_debug_addr (label, "prolog breakpoint addr",
 				   dosizeonly);
     }
@@ -896,8 +756,6 @@ write_rtnend (int rtnnum, int dosizeonly
   char label1[MAX_ARTIFICIAL_LABEL_BYTES];
   char label2[MAX_ARTIFICIAL_LABEL_BYTES];
   int totsize;
-  vms_func_ref fde = &func_table[rtnnum];
-  int corrected_rtnnum = fde->funcdef_number;
 
   totsize = 0;
 
@@ -912,8 +770,10 @@ write_rtnend (int rtnnum, int dosizeonly
   totsize += write_debug_data1 (rtnend.dst_b_rtnend_unused, "unused",
 				dosizeonly);
 
-  ASM_GENERATE_INTERNAL_LABEL (label1, FUNC_BEGIN_LABEL, corrected_rtnnum);
-  ASM_GENERATE_INTERNAL_LABEL (label2, FUNC_END_LABEL, corrected_rtnnum);
+  ASM_GENERATE_INTERNAL_LABEL (label1, FUNC_BEGIN_LABEL,
+			       funcnum_table [rtnnum]);
+  ASM_GENERATE_INTERNAL_LABEL (label2, FUNC_END_LABEL,
+			       funcnum_table [rtnnum]);
   totsize += write_debug_delta4 (label2, label1, "routine size", dosizeonly);
 
   return totsize;
@@ -1383,13 +1275,12 @@ vmsdbgout_ignore_block (const_tree block
   return retval;
 }
 
 /* Add an entry for function DECL into the func_table.  */
 
 static void
 vmsdbgout_begin_function (tree decl)
 {
   const char *name = XSTR (XEXP (DECL_RTL (decl), 0), 0);
-  vms_func_ref fde;
 
   if (write_symbols == VMS_AND_DWARF2_DEBUG)
     (*dwarf2_debug_hooks.begin_function) (decl);
@@ -1397,16 +1288,15 @@ vmsdbgout_begin_function (tree decl)
   if (func_table_in_use == func_table_allocated)
     {
       func_table_allocated += FUNC_TABLE_INCREMENT;
-      func_table
-        = (vms_func_ref) xrealloc (func_table,
-				   func_table_allocated * sizeof (vms_func_node));
+      funcnam_table = XRESIZEVEC (char *, funcnam_table, func_table_allocated);
+      funcnum_table = XRESIZEVEC (unsigned int, funcnum_table,
+				  func_table_allocated);
     }
 
   /* Add the new entry to the end of the function name table.  */
-  fde = &func_table[func_table_in_use++];
-  fde->vms_func_name = xstrdup (name);
-  fde->funcdef_number = current_function_funcdef_no;
-
+  funcnam_table[func_table_in_use] = xstrdup (name);
+  funcnum_table[func_table_in_use] = current_function_funcdef_no;
+  func_table_in_use++;
 }
 
 static char fullname_buff [4096];
@@ -1586,12 +1472,13 @@ vmsdbgout_init (const char *main_input_f
   /* Allocate the initial hunk of the file_info_table.  */
   file_info_table = XCNEWVEC (dst_file_info_entry, FILE_TABLE_INCREMENT);
   file_info_table_allocated = FILE_TABLE_INCREMENT;
-
-  /* Skip the first entry - file numbers begin at 1 */
+  /* Skip the first entry - file numbers begin at 1.  */
   file_info_table_in_use = 1;
 
-  func_table = (vms_func_ref) xcalloc (FUNC_TABLE_INCREMENT, sizeof (vms_func_node));
+  funcnam_table = XCNEWVEC (char *, FUNC_TABLE_INCREMENT);
+  funcnum_table = XCNEWVEC (unsigned int, FUNC_TABLE_INCREMENT);
   func_table_allocated = FUNC_TABLE_INCREMENT;
+  /* Skip the first entry.  */
   func_table_in_use = 1;
 
   /* Allocate the initial hunk of the line_info_table.  */

^ permalink raw reply	[flat|nested] 6+ messages in thread

* Re: [PATCH] Fix function numbering in vmsdbgout.c
  2010-06-26 21:41 [PATCH] Fix function numbering in vmsdbgout.c Douglas B Rupp
@ 2010-06-28 17:30 ` Richard Henderson
  2010-06-28 18:22   ` Douglas B Rupp
  0 siblings, 1 reply; 6+ messages in thread
From: Richard Henderson @ 2010-06-28 17:30 UTC (permalink / raw)
  To: Douglas B Rupp; +Cc: Jim Wilson, gcc-patches

On 06/26/2010 12:45 PM, Douglas B Rupp wrote:
> This patch fixes a bug in VMS Debug format with function numbering.
> 
> OK to commit?

> -      func_table
> -        = (vms_func_ref) xrealloc (func_table,
> -				   func_table_allocated * sizeof (vms_func_node));
> +      funcnam_table = XRESIZEVEC (char *, funcnam_table, func_table_allocated);
> +      funcnum_table = XRESIZEVEC (unsigned int, funcnum_table,
> +				  func_table_allocated);

Wouldn't it be just as easy to convert this to VEC instead of
re-implementing as a different kind of hand-managed array?


r~

^ permalink raw reply	[flat|nested] 6+ messages in thread

* Re: [PATCH] Fix function numbering in vmsdbgout.c
  2010-06-28 17:30 ` Richard Henderson
@ 2010-06-28 18:22   ` Douglas B Rupp
  2010-06-28 19:06     ` Richard Henderson
  0 siblings, 1 reply; 6+ messages in thread
From: Douglas B Rupp @ 2010-06-28 18:22 UTC (permalink / raw)
  To: Richard Henderson; +Cc: Jim Wilson, gcc-patches

Richard Henderson wrote:
> On 06/26/2010 12:45 PM, Douglas B Rupp wrote:
>> This patch fixes a bug in VMS Debug format with function numbering.
>>
>> OK to commit?
> 
>> -      func_table
>> -        = (vms_func_ref) xrealloc (func_table,
>> -				   func_table_allocated * sizeof (vms_func_node));
>> +      funcnam_table = XRESIZEVEC (char *, funcnam_table, func_table_allocated);
>> +      funcnum_table = XRESIZEVEC (unsigned int, funcnum_table,
>> +				  func_table_allocated);
> 
> Wouldn't it be just as easy to convert this to VEC instead of
> re-implementing as a different kind of hand-managed array?

You mean use GGC_RESIZEVEC as in dwarf2out.c?
Will do.

^ permalink raw reply	[flat|nested] 6+ messages in thread

* Re: [PATCH] Fix function numbering in vmsdbgout.c
  2010-06-28 18:22   ` Douglas B Rupp
@ 2010-06-28 19:06     ` Richard Henderson
  2010-06-29  9:27       ` [PATCH] Fix function numbering in vmsdbgout.c [2nd try] Douglas B Rupp
  0 siblings, 1 reply; 6+ messages in thread
From: Richard Henderson @ 2010-06-28 19:06 UTC (permalink / raw)
  To: Douglas B Rupp; +Cc: Jim Wilson, gcc-patches

On 06/28/2010 10:07 AM, Douglas B Rupp wrote:
> Richard Henderson wrote:
>> On 06/26/2010 12:45 PM, Douglas B Rupp wrote:
>>> This patch fixes a bug in VMS Debug format with function numbering.
>>>
>>> OK to commit?
>>
>>> -      func_table
>>> -        = (vms_func_ref) xrealloc (func_table,
>>> -                   func_table_allocated * sizeof (vms_func_node));
>>> +      funcnam_table = XRESIZEVEC (char *, funcnam_table,
>>> func_table_allocated);
>>> +      funcnum_table = XRESIZEVEC (unsigned int, funcnum_table,
>>> +                  func_table_allocated);
>>
>> Wouldn't it be just as easy to convert this to VEC instead of
>> re-implementing as a different kind of hand-managed array?
> 
> You mean use GGC_RESIZEVEC as in dwarf2out.c?
> Will do.

No, I mean using vec.h to manage the array.


r~

^ permalink raw reply	[flat|nested] 6+ messages in thread

* [PATCH] Fix function numbering in vmsdbgout.c [2nd try]
  2010-06-28 19:06     ` Richard Henderson
@ 2010-06-29  9:27       ` Douglas B Rupp
  2010-06-29 16:40         ` Richard Henderson
  0 siblings, 1 reply; 6+ messages in thread
From: Douglas B Rupp @ 2010-06-29  9:27 UTC (permalink / raw)
  To: Richard Henderson; +Cc: Jim Wilson, gcc-patches

[-- Attachment #1: Type: text/plain, Size: 169 bytes --]

Richard Henderson wrote:
> No, I mean using vec.h to manage the array.

Is this what you had in mind? If so do you want me to convert the other 
tables to VEC?

--Doug
@@ -1383,13 +1275,12 @@ vmsdbgout_ignore_block (const_tree block
 }
 
-/* Add an entry for function DECL into the func_table.  */
+/* Add an entry for function DECL into the funcnam_table.  */
 
 static void


[-- Attachment #2: 10.3debugalpha.dif.2nd_try.txt --]
[-- Type: text/plain, Size: 6027 bytes --]

2010-06-28  Douglas B Rupp  <rupp@gnat.com>

	* vmsdbgout.c (func_table): Replace with VEC func{nam,num}_tables.
	(funcnam_table): New static table.
	(funcnum_table): New static table.
	(write_rtnbeg): Write value saved in funcnum_table.
	(write_rtnend): Write value saved in funcnum_table.
	(vmsdbgout_begin_function): Save current function info in
	(vmsdbgout_init): Initialize func{nam,num}_tables. Minor reformatting.
	(vmsdbgout_finish): Iterate over funcnum_table.

--- gcc/vmsdbgout.c	2010-06-16 12:40:46.000000000 -0700
+++ gcc/vmsdbgout.c	2010-06-28 23:51:57.000000000 -0700
@@ -106,25 +106,13 @@ static unsigned int file_info_table_in_u
    table.  */
 #define FILE_TABLE_INCREMENT 64
 
-/* A structure to hold basic information for the VMS end
-   routine.  */
-
-typedef struct vms_func_struct
-{
-  const char *vms_func_name;
-  unsigned funcdef_number;
-}
-vms_func_node;
-
-typedef struct vms_func_struct *vms_func_ref;
-
-static unsigned int func_table_allocated;
-static unsigned int func_table_in_use;
-#define FUNC_TABLE_INCREMENT 256
-
-/* A pointer to the base of a table that contains frame description
-   information for each routine.  */
-static vms_func_ref func_table;
+typedef char *char_p;
+DEF_VEC_P(char_p);
+DEF_VEC_ALLOC_P(char_p,heap);
+
+static VEC(char_p,heap) *funcnam_table;
+static VEC(unsigned,heap) *funcnum_table;
+#define FUNC_TABLE_INITIAL 256
 
 /* Local pointer to the name of the main input file.  Initialized in
    vmsdbgout_init.  */
@@ -805,9 +793,8 @@ write_rtnbeg (int rtnnum, int dosizeonly
   char label[MAX_ARTIFICIAL_LABEL_BYTES];
   DST_ROUTINE_BEGIN rtnbeg;
   DST_PROLOG prolog;
-  vms_func_ref fde = &func_table[rtnnum];
 
-  rtnname = fde->vms_func_name;
+  rtnname = VEC_index (char_p, funcnam_table, rtnnum);
   rtnnamelen = strlen (rtnname);
   rtnentryname = concat (rtnname, "..en", NULL);
 
@@ -878,7 +865,9 @@ write_rtnbeg (int rtnnum, int dosizeonly
       totsize += write_debug_header (&prolog.dst_a_prolog_header, "prolog",
 				     dosizeonly);
 
-      ASM_GENERATE_INTERNAL_LABEL (label, FUNC_PROLOG_LABEL, fde->funcdef_number);
+      ASM_GENERATE_INTERNAL_LABEL
+        (label, FUNC_PROLOG_LABEL,
+	 VEC_index (unsigned, funcnum_table, rtnnum));
       totsize += write_debug_addr (label, "prolog breakpoint addr",
 				   dosizeonly);
     }
@@ -896,8 +885,6 @@ write_rtnend (int rtnnum, int dosizeonly
   char label1[MAX_ARTIFICIAL_LABEL_BYTES];
   char label2[MAX_ARTIFICIAL_LABEL_BYTES];
   int totsize;
-  vms_func_ref fde = &func_table[rtnnum];
-  int corrected_rtnnum = fde->funcdef_number;
 
   totsize = 0;
 
@@ -912,8 +899,12 @@ write_rtnend (int rtnnum, int dosizeonly
   totsize += write_debug_data1 (rtnend.dst_b_rtnend_unused, "unused",
 				dosizeonly);
 
-  ASM_GENERATE_INTERNAL_LABEL (label1, FUNC_BEGIN_LABEL, corrected_rtnnum);
-  ASM_GENERATE_INTERNAL_LABEL (label2, FUNC_END_LABEL, corrected_rtnnum);
+  ASM_GENERATE_INTERNAL_LABEL
+   (label1, FUNC_BEGIN_LABEL,
+    VEC_index (unsigned, funcnum_table, rtnnum));
+  ASM_GENERATE_INTERNAL_LABEL
+   (label2, FUNC_END_LABEL,
+    VEC_index (unsigned, funcnum_table, rtnnum));
   totsize += write_debug_delta4 (label2, label1, "routine size", dosizeonly);
 
   return totsize;
@@ -1383,30 +1374,20 @@ vmsdbgout_ignore_block (const_tree block
   return retval;
 }
 
 /* Add an entry for function DECL into the func_table.  */
 
 static void
 vmsdbgout_begin_function (tree decl)
 {
   const char *name = XSTR (XEXP (DECL_RTL (decl), 0), 0);
-  vms_func_ref fde;
 
   if (write_symbols == VMS_AND_DWARF2_DEBUG)
     (*dwarf2_debug_hooks.begin_function) (decl);
 
-  if (func_table_in_use == func_table_allocated)
-    {
-      func_table_allocated += FUNC_TABLE_INCREMENT;
-      func_table
-        = (vms_func_ref) xrealloc (func_table,
-				   func_table_allocated * sizeof (vms_func_node));
-    }
-
   /* Add the new entry to the end of the function name table.  */
-  fde = &func_table[func_table_in_use++];
-  fde->vms_func_name = xstrdup (name);
-  fde->funcdef_number = current_function_funcdef_no;
-
+  VEC_safe_push (char_p, heap, funcnam_table, xstrdup (name));
+  VEC_safe_push (unsigned, heap, funcnum_table,
+		 current_function_funcdef_no);
 }
 
 static char fullname_buff [4096];
@@ -1586,13 +1567,11 @@ vmsdbgout_init (const char *main_input_f
   /* Allocate the initial hunk of the file_info_table.  */
   file_info_table = XCNEWVEC (dst_file_info_entry, FILE_TABLE_INCREMENT);
   file_info_table_allocated = FILE_TABLE_INCREMENT;
-
-  /* Skip the first entry - file numbers begin at 1 */
+  /* Skip the first entry - file numbers begin at 1.  */
   file_info_table_in_use = 1;
 
-  func_table = (vms_func_ref) xcalloc (FUNC_TABLE_INCREMENT, sizeof (vms_func_node));
-  func_table_allocated = FUNC_TABLE_INCREMENT;
-  func_table_in_use = 1;
+  funcnam_table = VEC_alloc (char_p, heap, FUNC_TABLE_INITIAL);
+  funcnum_table = VEC_alloc (unsigned, heap, FUNC_TABLE_INITIAL);
 
   /* Allocate the initial hunk of the line_info_table.  */
   line_info_table = XCNEWVEC (dst_line_info_entry, LINE_INFO_TABLE_INCREMENT);
@@ -1679,7 +1658,7 @@ vmsdbgout_abstract_function (tree decl)
 static void
 vmsdbgout_finish (const char *main_input_filename ATTRIBUTE_UNUSED)
 {
-  unsigned int i;
+  unsigned int i, ifunc;
   int totsize;
 
   if (write_symbols == VMS_AND_DWARF2_DEBUG)
@@ -1699,7 +1678,7 @@ vmsdbgout_finish (const char *main_input
   ASM_OUTPUT_ALIGN (asm_out_file, 0);
 
   totsize = write_modbeg (1);
-  for (i = 1; i < func_table_in_use; i++)
+  for (i = 0; VEC_iterate (unsigned, funcnum_table, i, ifunc); i++)
     {
       totsize += write_rtnbeg (i, 1);
       totsize += write_rtnend (i, 1);
@@ -1707,7 +1686,7 @@ vmsdbgout_finish (const char *main_input
   totsize += write_pclines (1);
 
   write_modbeg (0);
-  for (i = 1; i < func_table_in_use; i++)
+  for (i = 0; VEC_iterate (unsigned, funcnum_table, i, ifunc); i++)
     {
       write_rtnbeg (i, 0);
       write_rtnend (i, 0);

^ permalink raw reply	[flat|nested] 6+ messages in thread

* Re: [PATCH] Fix function numbering in vmsdbgout.c [2nd try]
  2010-06-29  9:27       ` [PATCH] Fix function numbering in vmsdbgout.c [2nd try] Douglas B Rupp
@ 2010-06-29 16:40         ` Richard Henderson
  0 siblings, 0 replies; 6+ messages in thread
From: Richard Henderson @ 2010-06-29 16:40 UTC (permalink / raw)
  To: Douglas B Rupp; +Cc: Jim Wilson, gcc-patches

On 06/29/2010 12:13 AM, Douglas B Rupp wrote:
> Richard Henderson wrote:
>> No, I mean using vec.h to manage the array.
> 
> Is this what you had in mind? If so do you want me to convert the other
> tables to VEC?

Yes, this is what I had in mind.  This patch is ok.

If you'd like to convert the other tables, that would be great.


r~

^ permalink raw reply	[flat|nested] 6+ messages in thread

end of thread, other threads:[~2010-06-29 15:20 UTC | newest]

Thread overview: 6+ messages (download: mbox.gz / follow: Atom feed)
-- links below jump to the message on this page --
@@ -1383,30 +1374,20 @@ vmsdbgout_ignore_block (const_tree block
 }
 
-/* Add an entry for function DECL into the func_table.  */
+/* Add an entry for function DECL into the funcnam_table.  */
 
 static void
2010-06-26 21:41 [PATCH] Fix function numbering in vmsdbgout.c Douglas B Rupp
2010-06-28 17:30 ` Richard Henderson
2010-06-28 18:22   ` Douglas B Rupp
2010-06-28 19:06     ` Richard Henderson
2010-06-29  9:27       ` [PATCH] Fix function numbering in vmsdbgout.c [2nd try] Douglas B Rupp
2010-06-29 16:40         ` Richard Henderson

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for read-only IMAP folder(s) and NNTP newsgroup(s).